==> cursors_extended-neoforge-4.0.0%2B1.21.9-sources.jar: io/github/fishstiz/cursors_extended/gui/widget/CursorPreviewWidget.java <==
package io.github.fishstiz.cursors_extended.gui.widget;

import com.mojang.blaze3d.platform.cursor.CursorType;
import io.github.fishstiz.cursors_extended.CursorsExtended;
import io.github.fishstiz.cursors_extended.cursor.Cursor;
import io.github.fishstiz.cursors_extended.resource.AnimatedCursorTexture;
import io.github.fishstiz.cursors_extended.util.DrawUtil;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.GuiGraphics;
import net.minecraft.client.gui.components.Button;
import net.minecraft.client.input.MouseButtonEvent;
import net.minecraft.network.chat.CommonComponents;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class CursorPreviewWidget extends CursorWidget {
    private static final ResourceLocation BACKGROUND_128 = CursorsExtended.loc("textures/gui/background_dark_128.png");
    private static final Component PREVIEW_TEXT = Component.translatable("cursors_extended.options.preview");
    private static final int PREVIEW_TEXT_OFFSET = 4;
    private static final int PREVIEW_TEXT_COLOR = 0x7FFFFFFF; // 50% white
    private static final int RULER_COLOR = 0xFF00FF00; // green
    private static final int BACKGROUND_DISABLED = 0x7F000000; // 50% black
    private static final int BACKGROUND_OVERFLOW = 0xFF6A6A6A; // gray
    private static final int DEFAULT_BUTTON_SIZE = 20;
    private final @Nullable Button button;
    private final Font font;
    private boolean mouseEntered;

    public CursorPreviewWidget(@NotNull Cursor cursor, @NotNull Font font, @Nullable Button button) {
        super(CommonComponents.EMPTY, cursor, BACKGROUND_128);

        this.active = false;
        this.font = font;
        this.button = button;
    }

    public CursorPreviewWidget(@NotNull Cursor cursor, @NotNull Font font) {
        this(cursor, font, Button.builder(CommonComponents.EMPTY, b -> b.setFocused(false))
                .size(DEFAULT_BUTTON_SIZE, DEFAULT_BUTTON_SIZE)
                .build()
        );
    }

    @Override
    protected void renderBackground(@NotNull GuiGraphics guiGraphics) {
        super.renderBackground(guiGraphics);

        if (this.isOverflowing()) {
            guiGraphics.fill(this.getX(), this.getY(), this.getRight(), this.getBottom(), BACKGROUND_OVERFLOW);
        }
    }

    protected void updateMouseMoved() {
        if (this.isHovered()) {
            if (!mouseEntered && getCursor().getTexture() instanceof AnimatedCursorTexture animatedCursorTexture) {
                animatedCursorTexture.restartAnimation();
            }
            mouseEntered = true;
        } else {
            mouseEntered = false;
        }
    }

    @Override
    protected void renderWidget(@NotNull GuiGraphics guiGraphics, int mouseX, int mouseY, float partialTick) {
        Cursor cursor = this.getCursor();

        if (cursor.hasTexture()) {
            this.renderBackground(guiGraphics);
            if (cursor.isTextureEnabled()) {
                this.renderPreviewText(guiGraphics);
                this.renderTestButton(guiGraphics, mouseX, mouseY, partialTick);
                this.renderRuler(guiGraphics, mouseX, mouseY);
            }
        }
        if (!cursor.isTextureEnabled()) {
            guiGraphics.fill(this.getX(), this.getY(), this.getRight(), this.getBottom(), BACKGROUND_DISABLED);
        }

        this.renderBorder(guiGraphics);

        updateMouseMoved();
        if (this.isHovered()) guiGraphics.requestCursor(this.cursors_extended$cursorType(mouseX, mouseY));
    }

    protected void renderPreviewText(@NotNull GuiGraphics guiGraphics) {
        int width = this.font.width(PREVIEW_TEXT);
        int endX = this.getRight() - PREVIEW_TEXT_OFFSET;
        int endY = this.getBottom() - PREVIEW_TEXT_OFFSET;
        int startX = endX - width;
        int startY = endY - this.font.lineHeight;
        DrawUtil.drawScrollableTextLeftAlign(guiGraphics, this.font, PREVIEW_TEXT, startX, startY, endX, endY, PREVIEW_TEXT_COLOR, false);
    }

    protected void renderTestButton(@NotNull GuiGraphics guiGraphics, int mouseX, int mouseY, float partialTick) {
        if (this.button != null) {
            int buttonX = this.getX() + (this.getWidth() / 2 - this.button.getWidth() / 2);
            int buttonY = this.getY() + (this.getHeight() / 2 - this.button.getHeight() / 2);
            this.button.setPosition(buttonX, buttonY);
            this.button.render(guiGraphics, mouseX, mouseY, partialTick);
        }
    }

    @Override
    protected void renderRuler(@NotNull GuiGraphics guiGraphics, int mouseX, int mouseY) {
        if (this.isRenderRuler() && this.isMouseOver(mouseX, mouseY)) {
            guiGraphics.hLine(this.getX(), this.getRight() - 1, mouseY, RULER_COLOR);
            guiGraphics.vLine(mouseX, getY(), this.getBottom(), RULER_COLOR);
        }
    }

    @Override
    public boolean mouseClicked(MouseButtonEvent mouseButtonEvent, boolean doubleClicked) {
        if (this.button != null && this.button.isMouseOver(mouseButtonEvent.x(), mouseButtonEvent.y())) {
            this.button.mouseClicked(mouseButtonEvent, doubleClicked);
        }
        return false;
    }

    @Override
    public boolean isMouseOver(double mouseX, double mouseY) {
        return this.visible
               && mouseX >= (double) this.getX()
               && mouseY >= (double) this.getY()
               && mouseX < (double) this.getRight()
               && mouseY < (double) this.getBottom();
    }

    @Override
    protected float getCellWidth() {
        return this.getHeight() != this.getWidth() ? Math.max(super.getCellHeight(), super.getCellWidth()) : super.getCellWidth();
    }

    @Override
    protected float getCellHeight() {
        return this.getHeight() != this.getWidth() ? Math.max(super.getCellHeight(), super.getCellWidth()) : super.getCellHeight();
    }

    @Override
    public CursorType cursors_extended$cursorType(double mouseX, double mouseY) {
        return this.getCursor().cursorType();
    }
}

==> cursors_extended-neoforge-4.0.1%2B1.21.9-sources.jar: io/github/fishstiz/cursors_extended/gui/widget/CursorPreviewWidget.java <==
package io.github.fishstiz.cursors_extended.gui.widget;

import com.mojang.blaze3d.platform.cursor.CursorType;
import io.github.fishstiz.cursors_extended.CursorsExtended;
import io.github.fishstiz.cursors_extended.cursor.Cursor;
import io.github.fishstiz.cursors_extended.resource.texture.AnimatedCursorTexture;
import io.github.fishstiz.cursors_extended.util.DrawUtil;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.GuiGraphics;
import net.minecraft.client.gui.components.Button;
import net.minecraft.client.input.MouseButtonEvent;
import net.minecraft.network.chat.CommonComponents;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class CursorPreviewWidget extends CursorWidget {
    private static final ResourceLocation BACKGROUND_128 = CursorsExtended.loc("textures/gui/background_dark_128.png");
    private static final Component PREVIEW_TEXT = Component.translatable("cursors_extended.options.preview");
    private static final int PREVIEW_TEXT_OFFSET = 4;
    private static final int PREVIEW_TEXT_COLOR = 0x7FFFFFFF; // 50% white
    private static final int RULER_COLOR = 0xFF00FF00; // green
    private static final int BACKGROUND_DISABLED = 0x7F000000; // 50% black
    private static final int BACKGROUND_OVERFLOW = 0xFF6A6A6A; // gray
    private static final int DEFAULT_BUTTON_SIZE = 20;
    private final @Nullable Button button;
    private final Font font;

    public CursorPreviewWidget(@NotNull Cursor cursor, @NotNull Font font, @Nullable Button button) {
        super(CommonComponents.EMPTY, cursor, BACKGROUND_128);

        this.active = false;
        this.font = font;
        this.button = button;
    }

    public CursorPreviewWidget(@NotNull Cursor cursor, @NotNull Font font) {
        this(cursor, font, Button.builder(CommonComponents.EMPTY, b -> b.setFocused(false))
                .size(DEFAULT_BUTTON_SIZE, DEFAULT_BUTTON_SIZE)
                .build()
        );
    }

    @Override
    protected void renderBackground(@NotNull GuiGraphics guiGraphics) {
        super.renderBackground(guiGraphics);

        if (this.isOverflowing()) {
            guiGraphics.fill(this.getX(), this.getY(), this.getRight(), this.getBottom(), BACKGROUND_OVERFLOW);
        }
    }

    @Override
    protected void renderWidget(@NotNull GuiGraphics guiGraphics, int mouseX, int mouseY, float partialTick) {
        Cursor cursor = this.getCursor();

        if (cursor.hasTexture()) {
            this.renderBackground(guiGraphics);
            if (cursor.isTextureEnabled()) {
                this.renderPreviewText(guiGraphics);
                this.renderTestButton(guiGraphics, mouseX, mouseY, partialTick);
                this.renderRuler(guiGraphics, mouseX, mouseY);
            }
        }
        if (!cursor.isTextureEnabled()) {
            guiGraphics.fill(this.getX(), this.getY(), this.getRight(), this.getBottom(), BACKGROUND_DISABLED);
        }

        this.renderBorder(guiGraphics);

        if (this.isHovered()) guiGraphics.requestCursor(this.cursors_extended$cursorType(mouseX, mouseY));
    }

    protected void renderPreviewText(@NotNull GuiGraphics guiGraphics) {
        int width = this.font.width(PREVIEW_TEXT);
        int endX = this.getRight() - PREVIEW_TEXT_OFFSET;
        int endY = this.getBottom() - PREVIEW_TEXT_OFFSET;
        int startX = endX - width;
        int startY = endY - this.font.lineHeight;
        DrawUtil.drawScrollableTextLeftAlign(guiGraphics, this.font, PREVIEW_TEXT, startX, startY, endX, endY, PREVIEW_TEXT_COLOR, false);
    }

    protected void renderTestButton(@NotNull GuiGraphics guiGraphics, int mouseX, int mouseY, float partialTick) {
        if (this.button != null) {
            int buttonX = this.getX() + (this.getWidth() / 2 - this.button.getWidth() / 2);
            int buttonY = this.getY() + (this.getHeight() / 2 - this.button.getHeight() / 2);
            this.button.setPosition(buttonX, buttonY);
            this.button.render(guiGraphics, mouseX, mouseY, partialTick);
        }
    }

    @Override
    protected void renderRuler(@NotNull GuiGraphics guiGraphics, int mouseX, int mouseY) {
        if (this.isRenderRuler() && this.isMouseOver(mouseX, mouseY)) {
            guiGraphics.hLine(this.getX(), this.getRight() - 1, mouseY, RULER_COLOR);
            guiGraphics.vLine(mouseX, getY(), this.getBottom(), RULER_COLOR);
        }
    }

    @Override
    public boolean mouseClicked(MouseButtonEvent mouseButtonEvent, boolean doubleClicked) {
        if (this.button != null && this.button.isMouseOver(mouseButtonEvent.x(), mouseButtonEvent.y())) {
            this.button.mouseClicked(mouseButtonEvent, doubleClicked);
            if (this.getCursor().getTexture() instanceof AnimatedCursorTexture animatedCursorTexture) {
                animatedCursorTexture.restartAnimation();
            }
        }
        return false;
    }

    @Override
    public boolean isMouseOver(double mouseX, double mouseY) {
        return this.visible
               && mouseX >= (double) this.getX()
               && mouseY >= (double) this.getY()
               && mouseX < (double) this.getRight()
               && mouseY < (double) this.getBottom();
    }

    @Override
    protected float getCellWidth() {
        return this.getHeight() != this.getWidth() ? Math.max(super.getCellHeight(), super.getCellWidth()) : super.getCellWidth();
    }

    @Override
    protected float getCellHeight() {
        return this.getHeight() != this.getWidth() ? Math.max(super.getCellHeight(), super.getCellWidth()) : super.getCellHeight();
    }

    @Override
    public CursorType cursors_extended$cursorType(double mouseX, double mouseY) {
        return this.getCursor().cursorType();
    }
}
